/* 
 * Copyright (c) 2008 Jesper Karlsson & David Gustafsson
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif
#include <gtk/gtk.h>
#include <glib.h>

#include "xfbib-multiple-input.h"
#include "xfbib-input-dialog.h"
#include "xfbib-bibtex-string.h"
#include "xfbib-bibtex-value.h"
#include "xfbib-string.h"
#include "xfbib-integer.h"
#include "xfbib-state.h"

struct _XfbibMultipleInput
{
	GtkVBox parent;
	GtkWidget *tree_view;
	GtkWidget *add_button;
	GtkWidget *remove_button;
};

typedef struct _XfbibMultipleInputClass
{
	GtkVBoxClass parent;
} XfbibMultipleInputClass;

static void xfbib_multiple_input_class_init(XfbibMultipleInputClass *klass);

static void xfbib_multiple_input_init(XfbibMultipleInput *instance);
static void xfbib_multiple_input_finalize(GObject *obj);
static void cb_cell_edited_event(GtkCellRendererText *renderer, char *path,
		char *new_text, GtkTreeView *treeview);
static void cb_add_button_clicked_event(GtkWidget *ptr);
static void cb_remove_button_clicked_event(GtkWidget *ptr);

static GObjectClass *xfbib_multiple_input_parent_class = NULL;

static void cb_cell_edited_event (GtkCellRendererText *renderer, char *path,
		char *text, GtkTreeView *treeview)
{
	GtkTreeIter iter;
	GtkTreeModel *model;
	if (strcasecmp (text, "") != 0) {
  		model = gtk_tree_view_get_model (treeview);
		if (gtk_tree_model_get_iter_from_string (model, &iter, path))
			gtk_list_store_set (GTK_LIST_STORE (model), &iter, 0, text, -1);
  	}
}

static void cb_add_button_clicked_event(GtkWidget *tree_view)
{
	g_print("cb_add_button_clicked_event\n");
	GtkWidget *dialog;
	GtkListStore *list;
	GtkTreeIter iter;
	GtkTreeViewColumn *column;
	const gchar *string_key, *text;
	gchar *str;
	
	column = gtk_tree_view_get_column(GTK_TREE_VIEW(tree_view), 0);
	str = g_strdup_printf(_("Add %s"), g_ascii_strdown(gtk_tree_view_column_get_title(column), -1));
	dialog = xfbib_input_dialog_new(str, gtk_tree_view_column_get_title(column));
	g_free(str);

	if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_OK) {
		string_key = xfbib_input_dialog_get_string_key(XFBIB_INPUT_DIALOG(dialog));
		text = xfbib_input_dialog_get_text(XFBIB_INPUT_DIALOG(dialog));
		if ((strcasecmp (string_key, "") != 0) || (strcasecmp (text, "") != 0)) {
			list = GTK_LIST_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW(tree_view)));
			gtk_list_store_append(list, &iter);
			if (strcasecmp (string_key, "") != 0) {
				gtk_list_store_set(list, &iter, 0, string_key, 1, text, -1);
			} else {
				gtk_list_store_set(list, &iter, 0, text, -1);
			}
			/* TODO: Update colors */
		}
	}
	gtk_widget_destroy(dialog);
}

static void cb_remove_button_clicked_event(GtkWidget *tree_view)
{
	g_print("cb_remove_button_clicked_event\n");
	GtkTreeModel *model;
	GtkTreeIter iter;
	GtkTreeSelection *selection;
	
	model = gtk_tree_view_get_model(GTK_TREE_VIEW(tree_view));
	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(tree_view));
	if (gtk_tree_selection_get_selected(selection, &model, &iter)) {
		gtk_list_store_remove(GTK_LIST_STORE(model), &iter);
		/* TODO: Update colors */
	}
}

GType
xfbib_multiple_input_get_type (void)
{
	static GType type = 0;
	if (type == 0) {
		static const GTypeInfo info = {
			sizeof (XfbibMultipleInputClass),
			NULL,   /* base_init */
			NULL,   /* base_finalize */
			(GClassInitFunc) xfbib_multiple_input_class_init,   /* class_init */
			NULL,   /* class_finalize */
			NULL,   /* class_data */
			sizeof (XfbibMultipleInput),
			0,      /* n_preallocs */
			(GInstanceInitFunc) xfbib_multiple_input_init    /* instance_init */
		};
		type = g_type_register_static (GTK_TYPE_VBOX, "XfbibMultipleInput", &info, 0);
	}
	return type;
}
	
static void
xfbib_multiple_input_class_init(XfbibMultipleInputClass *klass)
{
	GObjectClass *object_class = (GObjectClass *)klass;
	
	xfbib_multiple_input_parent_class = g_type_class_peek_parent (klass); 

	object_class->finalize = xfbib_multiple_input_finalize;
}

static void
xfbib_multiple_input_init(XfbibMultipleInput *instance)
{
	GtkWidget *button_box;
	GtkListStore *list;
	GtkCellRenderer *renderer;
	GtkTreeViewColumn *column;
	GtkWidget *scrolled_window;

	gtk_box_set_spacing(GTK_BOX(instance), 12);

	button_box = gtk_hbox_new(FALSE, 12);
	
	/* Tree view */
	instance->tree_view = gtk_tree_view_new();

	renderer = gtk_cell_renderer_text_new();
	g_object_set(renderer, "editable", TRUE, NULL);
	g_signal_connect(renderer, "edited", (GCallback) cb_cell_edited_event, instance->tree_view);

	column = gtk_tree_view_column_new_with_attributes(_("Title"), renderer, "text", 0, NULL);
	gtk_tree_view_column_set_resizable(GTK_TREE_VIEW_COLUMN(column), TRUE);
	gtk_tree_view_column_set_min_width(GTK_TREE_VIEW_COLUMN(column), 100);
   	gtk_tree_view_append_column(GTK_TREE_VIEW(instance->tree_view), column);

	list = gtk_list_store_new (2, G_TYPE_STRING, G_TYPE_STRING);

	gtk_tree_view_set_model(GTK_TREE_VIEW(instance->tree_view), GTK_TREE_MODEL(list));
	g_object_unref(list);

	scrolled_window = gtk_scrolled_window_new(NULL, NULL);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
			GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (scrolled_window), GTK_SHADOW_OUT);

	instance->add_button = gtk_button_new_from_stock(GTK_STOCK_ADD);
	g_signal_connect_swapped(G_OBJECT(instance->add_button), "clicked",
			G_CALLBACK(cb_add_button_clicked_event), instance->tree_view);

	instance->remove_button = gtk_button_new_from_stock(GTK_STOCK_REMOVE);
	g_signal_connect_swapped(G_OBJECT(instance->remove_button), "clicked",
			G_CALLBACK(cb_remove_button_clicked_event), instance->tree_view);

	gtk_box_pack_start(GTK_BOX(button_box), instance->add_button, TRUE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(button_box), instance->remove_button, TRUE, FALSE, 0);
	
	gtk_container_add (GTK_CONTAINER (scrolled_window), instance->tree_view);

	gtk_box_pack_start(GTK_BOX(instance), scrolled_window, TRUE, TRUE, 0);
	gtk_box_pack_end(GTK_BOX(instance), button_box, FALSE, TRUE, 0);
	gtk_widget_show_all(GTK_WIDGET(instance));
}

static void
xfbib_multiple_input_finalize(GObject *obj)
{
	G_OBJECT_CLASS(xfbib_multiple_input_parent_class)->finalize(obj);
}

GtkWidget *
xfbib_multiple_input_new(gchar *title, gchar *tooltip)
{
	GtkTreeViewColumn *column;
	XfbibMultipleInput *multiple_input;
	multiple_input = g_object_new(XFBIB_TYPE_MULTIPLE_INPUT, NULL);
	
	column = gtk_tree_view_get_column(GTK_TREE_VIEW(multiple_input->tree_view), 0);
	gtk_tree_view_column_set_title(column, title);

	gtk_widget_set_tooltip_text(multiple_input->tree_view, tooltip);

	return GTK_WIDGET(multiple_input);
}

void
xfbib_multiple_input_set_text(XfbibMultipleInput *multiple_input, const gchar *text)
{
	GtkTreeModel *model;
	GtkTreeIter iter;
	GtkTreeViewColumn *column;
	gchar **split = NULL;
	const gchar *title;
	gint n;

	model = gtk_tree_view_get_model(GTK_TREE_VIEW (multiple_input->tree_view));
	column = gtk_tree_view_get_column(GTK_TREE_VIEW(multiple_input->tree_view), 0);
	title = gtk_tree_view_column_get_title(column);

	if(strcasecmp(title, "author") == 0) {
		split = g_strsplit(text, " and ", 0);
		for (n = 0; split[n] != NULL; n++) {
			gtk_list_store_append(GTK_LIST_STORE (model), &iter);
			gtk_list_store_set(GTK_LIST_STORE (model), &iter, 0, split[n], -1);
		}
	} else {
		gtk_list_store_append(GTK_LIST_STORE (model), &iter);
		gtk_list_store_set(GTK_LIST_STORE (model), &iter, 0, text, -1);
	}
}

void
xfbib_multiple_input_set(XfbibMultipleInput *multiple_input, GList *gl)
{
	GtkTreeModel *model;
	GtkTreeIter iter;
	gint n, len;
	GObject *obj;

	model = gtk_tree_view_get_model(GTK_TREE_VIEW (multiple_input->tree_view));
	
	len = g_list_length(gl);
	for (n = 0; n < len; n++) {
		gtk_list_store_append(GTK_LIST_STORE (model), &iter);
		obj = g_list_nth_data(gl, n);
		if(XFBIB_IS_STRING(obj)) {
			gtk_list_store_set(GTK_LIST_STORE (model), &iter, 0, xfbib_string_get(XFBIB_STRING(obj)), -1);
		} else if(XFBIB_IS_BIBTEX_STRING(obj)){
			gtk_list_store_set(GTK_LIST_STORE (model), &iter, 0, xfbib_bibtex_string_get_variable(XFBIB_BIBTEX_STRING(obj)),
					1, xfbib_bibtex_value_get_str(xfbib_bibtex_string_get_value(XFBIB_BIBTEX_STRING(obj))), -1);
		} else if(XFBIB_IS_INTEGER(obj)) {
			gtk_list_store_set(GTK_LIST_STORE (model), &iter, 0, xfbib_integer_get(XFBIB_INTEGER(obj)), -1);
		}
	}
}

const gchar *
xfbib_multiple_input_get_nth_row_ith_column(XfbibMultipleInput *multiple_input, gint row, gint column)
{
	GtkTreeModel *model;
	GtkTreeIter iter;
	gint n = 0;
	const gchar *tmp;

	model = gtk_tree_view_get_model(GTK_TREE_VIEW (multiple_input->tree_view));
	if(gtk_tree_model_get_iter_first(model, &iter)) {
		while((n < row) && gtk_tree_model_iter_next(model, &iter)) {
			n++;
		}
		if(n == row) {
			gtk_tree_model_get(model, &iter, column, &tmp, -1);
			return tmp;
		}
	}
	return NULL;
}

const gchar *
xfbib_multiple_input_get_str(XfbibMultipleInput *multiple_input)
{
	GString *strbuf;
	const gchar *str;
	int n;

	for(n = 0; xfbib_multiple_input_get_nth_row_ith_column(multiple_input, n, 0) != NULL; n++) {
		str = xfbib_multiple_input_get_nth_row_ith_column(multiple_input, n, 1);
		if(str != NULL) {
			/* Atleast one bibtex string found, needs to be handled by xfbib_multiple_input_get */
			return NULL;
		}
	}
	/* Only text in multiple input, create one long string and return it */
	strbuf = g_string_new("");
	for(n = 0; (str = xfbib_multiple_input_get_nth_row_ith_column(multiple_input, n, 0)) != NULL; n++) {
		if(n != 0) {
			g_string_append(strbuf, " and ");
		}
		g_string_append(strbuf, str);
	}
	return g_string_free(strbuf, FALSE);
}

GList *
xfbib_multiple_input_get(XfbibMultipleInput *multiple_input)
{ 
	GList *list = NULL;
	XfbibString *str;
	XfbibState *state;
	XfbibBibtexString *string;
	XfbibBibtexValue *value;
	const gchar *bibstr, *text;
	gint n;

	state = xfbib_state_new();

	for(n = 0; (text = xfbib_multiple_input_get_nth_row_ith_column(multiple_input, n, 0)) != NULL; n++) {
		bibstr = xfbib_multiple_input_get_nth_row_ith_column(multiple_input, n, 1);
		if(bibstr != NULL) {
			string = xfbib_bibtex_string_new(G_OBJECT(xfbib_state_get_bibtex_elements(state)));
			value = xfbib_bibtex_value_new(G_OBJECT(string));

			xfbib_bibtex_string_set_variable(string, bibstr);
			xfbib_bibtex_value_set_str(value, text);
			xfbib_bibtex_string_set_value(string, value);

			list = g_list_append(list, G_OBJECT(string));
		} else {
			str = xfbib_string_new(G_OBJECT(xfbib_state_get_bibtex_elements(state)));
			xfbib_string_set(str, text);
			list = g_list_append(list, G_OBJECT(str));
		}
	}
	return list;
}

